package hr.webtech.pay2.api;

import android.support.annotation.NonNull;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.io.IOException;

import hr.webtech.pay2.util.Objects;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

/**
 * Created by jasminsuljic on 19/12/16.
 * pgw-android
 */

/**
 * PgwModule
 * <p>
 * Provides {@link PgwApi}.
 * <p>
 * Every request failure is parsed via {@link RxErrorHandlingCallAdapterFactory} and then if possible
 * converted to {@link ApiErrorException}
 */
public final class PgwModule {
//    private static final String API_URL = "http://149.202.150.255/pgw";
    private static final String API_URL = "https://mobile.webteh.hr/pgw";
    private static final String API_VERSION = null;
    public static final String API_URL_PAY_CVV = buildUrl(API_URL, API_VERSION, "default/pay-cvv/%s");
    public static final String API_URL_ADD_CARD = buildUrl(API_URL, API_VERSION, "default/add-card/%s");
    public static final String API_URL_ADD_PAYMENT_METHOD = buildUrl(API_URL, API_VERSION, "default/add-payment-method/%s");
    private static final String API_AUTH_KEY_QUERY = "auth_token";
    private final PgwApi api;
    private final String authToken;
    private final Gson gson;
    private boolean isDebug;

    private static String buildUrl(String... tokens) {
        StringBuilder sb = new StringBuilder();
        String delimeter = "/";
        boolean firstTime = true;
        for (Object token : tokens) {

            if (token == null) {
                continue;
            }

            if (firstTime) {
                firstTime = false;
            } else {
                sb.append(delimeter);
            }

            sb.append(token);
        }
        return sb.toString();

    }

    /**
     * @param authToken Merchant authToken provided from PGW Proxy
     * @param isDebug   If true {@link HttpLoggingInterceptor} will be added
     */
    public PgwModule(@NonNull String authToken, boolean isDebug) {
        Objects.requireNonNull(authToken, "authToken == null");
        this.isDebug = isDebug;
        this.authToken = authToken;
        this.gson = new GsonBuilder()
                .create();
        this.api = provideApi(provideRetrofit(gson, provideOkHttpClient(), provideRxErrorHandlingCallAdapterFactory()));
    }

    private OkHttpClient provideOkHttpClient() {

        final OkHttpClient.Builder builder = new OkHttpClient.Builder();

        builder.addInterceptor(new Interceptor() {
            @Override
            public Response intercept(Chain chain) throws IOException {
                Request original = chain.request();
                return chain
                        .proceed(original.newBuilder()
                                .url(
                                        original
                                                .url()
                                                .newBuilder()
                                                .addQueryParameter(API_AUTH_KEY_QUERY, authToken)
                                                .build()
                                ).build());
            }
        });

        if (isDebug) {
            HttpLoggingInterceptor interceptor = new HttpLoggingInterceptor();
            interceptor.setLevel(HttpLoggingInterceptor.Level.BODY);
            return builder.addInterceptor(interceptor).build();
        }

        return builder.build();
    }

    private Retrofit provideRetrofit(Gson gson, OkHttpClient okHttpClient, RxErrorHandlingCallAdapterFactory rxErrorHandlingCallAdapterFactory) {
        return new Retrofit.Builder()
                .baseUrl(API_URL + "/api/")
                .addConverterFactory(GsonConverterFactory.create(gson))
                .addCallAdapterFactory(rxErrorHandlingCallAdapterFactory)
                .client(okHttpClient)
                .build();
    }


    private RxErrorHandlingCallAdapterFactory provideRxErrorHandlingCallAdapterFactory() {
        return RxErrorHandlingCallAdapterFactory.create();
    }

    private PgwApi provideApi(Retrofit retrofit) {
        return new PgwApiProxy(retrofit.create(PgwRestApi.class), gson);
    }

    public PgwApi getApi() {
        return api;
    }

}
