package hr.webtech.pay2.view;

import android.os.Handler;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import hr.webtech.pay2.data.ApiError;
import hr.webtech.pay2.data.ApiErrors;
import hr.webtech.pay2.data.BusinessError;
import hr.webtech.pay2.util.InputResolver;
import hr.webtech.pay2.util.Objects;

import static hr.webtech.pay2.view.WebtehWebView.ResolvedUrlResult;

/**
 * Created by jasminsuljic on 20/12/16.
 * pgw-android
 */
final class RedirectResolver {


    private final Handler handler;
    private final int timeoutHandlerPeriod = 500;
    private final Map<String, WebtehWebView.ResolvedUrlResult> resolvedUrlResultMap = new HashMap<>();
    private final Map<WebtehWebView.ResolvedUrlResult, RedirectResolverRunnable> resolvedUrlRunnableMap = new HashMap<>();
    private final InputResolver inputResolver;
    private final Gson gson;
    private final long timeout;
    private boolean transactionInProgress;
    private long timePassed;
    private final SharedLifecycleDelegate sharedLifecycleDelegate;

    @Nullable
    private Runnable timeoutRunnable;

    /**
     * Instantiates a new Redirect resolver.
     *
     * @param transactionLifecycleAdapter the transaction lifecycle
     * @param inputResolver               the input resolver
     * @param gson                        the gson
     * @param timeout                     the timeout
     */
    RedirectResolver(@NonNull TransactionLifecycleAdapter transactionLifecycleAdapter,
                     @NonNull InputResolver inputResolver,
                     Gson gson,
                     long timeout
    ) {

        if (timeout < 2 * timeoutHandlerPeriod) {
            throw new IllegalArgumentException("Timeout must be greater or equal = " + 2 * timeout);
        }

        Objects.requireNonNull(transactionLifecycleAdapter);

        this.inputResolver = Objects.requireNonNull(inputResolver);
        this.gson = gson;
        this.timeout = timeout;
        this.handler = new Handler();

        final TransactionLifecycleDelegate delegate = transactionLifecycleAdapter.getDelegate();

        this.sharedLifecycleDelegate = delegate;

        addResolvedUrlItems();
        addResolvedUrlRunnable(transactionLifecycleAdapter, delegate);
    }

    /**
     * Instantiates a new Redirect resolver.
     *
     * @param addPaymentMethodLifecycleAdapter the transaction lifecycle
     * @param inputResolver                    the input resolver
     * @param gson                             the gson
     * @param timeout                          the timeout
     */
    RedirectResolver(@NonNull AddPaymentMethodLifecycleAdapter addPaymentMethodLifecycleAdapter,
                     @NonNull InputResolver inputResolver,
                     Gson gson,
                     long timeout
    ) {

        if (timeout < 2 * timeoutHandlerPeriod) {
            throw new IllegalArgumentException("Timeout must be greater or equal = " + 2 * timeout);
        }

        Objects.requireNonNull(addPaymentMethodLifecycleAdapter);

        this.inputResolver = Objects.requireNonNull(inputResolver);
        this.gson = gson;
        this.timeout = timeout;
        this.handler = new Handler();

        final AddPaymentMethodLifecycleDelegate delegate = addPaymentMethodLifecycleAdapter.getDelegate();

        this.sharedLifecycleDelegate = delegate;

        addResolvedUrlItems();
        addResolvedUrlRunnable(addPaymentMethodLifecycleAdapter, delegate);
    }

    private void addResolvedUrlRunnable(final TransactionLifecycleAdapter transactionLifecycleAdapter, TransactionLifecycleDelegate delegate) {

        final StartedTransactionResolverRunnable startedTransactionResolverRunnable = new StartedTransactionResolverRunnable(sharedLifecycleDelegate);
        final ThreeDSResolverRunnable threeDSResolverRunnable = new ThreeDSResolverRunnable(delegate);

        final SuccessTransactionResolverRunnable successTransactionResolverRunnable = new SuccessTransactionResolverRunnable(delegate, gson);
        final TimeoutResolverRunnable timeoutResolverRunnable = new TimeoutResolverRunnable(delegate, new Runnable() {
            @Override
            public void run() {
                transactionLifecycleAdapter.timeout();
            }
        });
        final TransactionFailureResolverRunnable transactionFailureResolverRunnable = new TransactionFailureResolverRunnable(delegate);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.STARTED_TRANSACTION, startedTransactionResolverRunnable);
        resolvedUrlRunnableMap.put(ResolvedUrlResult.STARTED_TRANSACTION_3DS, startedTransactionResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.TRANSACTION_SUCCESS, successTransactionResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.TRANSACTION_FAILURE, transactionFailureResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.THREE_DS, threeDSResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.TIMEOUT, timeoutResolverRunnable);

    }

    private void addResolvedUrlRunnable(final AddPaymentMethodLifecycleAdapter addPaymentMethodLifecycleAdapter, AddPaymentMethodLifecycleDelegate delegate) {

        final StartedTransactionResolverRunnable startedTransactionResolverRunnable = new StartedTransactionResolverRunnable(delegate);
        final ThreeDSResolverRunnable threeDSResolverRunnable = new ThreeDSResolverRunnable(delegate);
        final AddPaymentMethodSuccessRunnable addPaymentMethodSuccessRunnable = new AddPaymentMethodSuccessRunnable(delegate, gson);
        final TransactionFailureResolverRunnable transactionFailureResolverRunnable = new TransactionFailureResolverRunnable(delegate);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.ADD_PAYMENT_METHOD_STARTED_TRANSACTION, startedTransactionResolverRunnable);
        resolvedUrlRunnableMap.put(ResolvedUrlResult.ADD_PAYMENT_METHOD_STARTED_TRANSACTION_3DS, startedTransactionResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.ADD_PAYMENT_METHOD_SUCCESS, addPaymentMethodSuccessRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.ADD_PAYMENT_METHOD_FAILURE, transactionFailureResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.ADD_PAYMENT_METHOD_THREE_DS, threeDSResolverRunnable);

        resolvedUrlRunnableMap.put(ResolvedUrlResult.TIMEOUT, new TimeoutResolverRunnable(delegate, new Runnable() {
            @Override
            public void run() {
                addPaymentMethodLifecycleAdapter.timeout();
            }
        }));
    }

    private void addResolvedUrlItems() {
        resolvedUrlResultMap.put("/api/add-card", ResolvedUrlResult.STARTED_TRANSACTION);
        resolvedUrlResultMap.put("/api/pay-cvv", ResolvedUrlResult.STARTED_TRANSACTION);
        resolvedUrlResultMap.put("/api/pares", ResolvedUrlResult.STARTED_TRANSACTION_3DS);
        resolvedUrlResultMap.put("/default/3ds", ResolvedUrlResult.THREE_DS);
        resolvedUrlResultMap.put("/api/success", ResolvedUrlResult.TRANSACTION_SUCCESS);
        resolvedUrlResultMap.put("/api/error", ResolvedUrlResult.TRANSACTION_FAILURE);

        resolvedUrlResultMap.put("/api/process-add-payment-method", ResolvedUrlResult.ADD_PAYMENT_METHOD_STARTED_TRANSACTION);
        resolvedUrlResultMap.put("/api/add-payment-method-pares", ResolvedUrlResult.ADD_PAYMENT_METHOD_STARTED_TRANSACTION_3DS);
        resolvedUrlResultMap.put("/default/add-payment-method-3ds", ResolvedUrlResult.ADD_PAYMENT_METHOD_THREE_DS);
        resolvedUrlResultMap.put("/api/add-payment-method-success", ResolvedUrlResult.ADD_PAYMENT_METHOD_SUCCESS);
        resolvedUrlResultMap.put("/api/add-payment-method-error", ResolvedUrlResult.ADD_PAYMENT_METHOD_FAILURE);
    }

    /**
     * Returns true if request is resolved
     *
     * @param request the request
     * @return webteh web view . resolved url result
     */
    ResolvedUrlResult resolve(String request) {

        final Set<Map.Entry<String, WebtehWebView.ResolvedUrlResult>> entries = resolvedUrlResultMap.entrySet();

        for (Map.Entry<String, WebtehWebView.ResolvedUrlResult> entry : entries) {
            if (request.contains(entry.getKey())) {
                return entry.getValue();
            }
        }

        return ResolvedUrlResult.UNKNOWN;
    }

    /**
     * Notify delegate based on {@link ResolvedUrlResult}
     *
     * @param resolve the resolve resolved url result
     * @param request the request Url request
     */
    void notifyDelegate(ResolvedUrlResult resolve, @Nullable String request) {
        final String[] parts = request == null ? new String[]{} : request.split("/");
        final String inputParam = parts.length == 0 ? null : inputResolver.decode(parts[parts.length - 1]);

        sharedLifecycleDelegate.logTransactionInfo(String.format("notifyDelegate: resolvedCode %d", resolve.ordinal()));

        if (resolvedUrlRunnableMap.containsKey(resolve)) {
            resolvedUrlRunnableMap.get(resolve).run(resolve, request, this, inputParam);
        } else {
            sharedLifecycleDelegate.logTransactionInfo(String.format("notifyDelegate runnable not available: resolvedCode %d", resolve.ordinal()));
        }
    }

    /**
     * Helper method for resolving transaction failure.
     * <p>
     * Tries parsing input parameter as BusinessError. If successful {@link TransactionLifecycleDelegate#handleTransactionError(ApiError)}
     * will be invoked.
     * <p>
     * If not {@link ApiError} instance will be created from inputParameter and {@link TransactionLifecycleDelegate#handleTransactionError(ApiError)}
     * will be invoked.
     *
     * @param inputParam
     */
    void handleTransactionFailure(@Nullable String inputParam) {

        try {
            final BusinessError businessError = gson.fromJson(inputParam, BusinessError.class);
            if (businessError != null && businessError.isValid()) {
                sharedLifecycleDelegate.logTransactionInfo(String.format("Transaction failure %s", businessError));
                sharedLifecycleDelegate.handleTransactionError(businessError);
            } else {
                final ApiError apiError = gson.fromJson(inputParam, ApiError.class);
                sharedLifecycleDelegate.logTransactionInfo(String.format("Transaction failure %s", apiError));
                sharedLifecycleDelegate.handleTransactionError(apiError);
            }
        } catch (JsonSyntaxException exception) {
            sharedLifecycleDelegate.logTransactionInfo(String.format("Transaction failure %s", exception.getMessage()));
            sharedLifecycleDelegate.handleTransactionError(ApiErrors.libraryParsingError(exception));
        }
    }

    void stopTimeoutTracking() {
        if (timeoutRunnable != null) {
            sharedLifecycleDelegate.logTransactionInfo("Stoping timeout tracking");
            handler.removeCallbacks(this.timeoutRunnable);
            timeoutRunnable = null;
        }
    }

    void startTimeoutTracking() {

        sharedLifecycleDelegate.logTransactionInfo("Starting timeout tracking");
        this.timeoutRunnable = new Runnable() {
            @Override
            public void run() {
                timePassed += timeoutHandlerPeriod;
                if (timePassed >= timeout) {
                    timePassed = 0;
                    sharedLifecycleDelegate.logTransactionInfo(String.format("Transaction timeout with timeout %d", timeout));
                    notifyDelegate(ResolvedUrlResult.TIMEOUT, null);
                } else {
                    handler.postDelayed(timeoutRunnable, timeoutHandlerPeriod);
                }
            }
        };

        handler.postDelayed(timeoutRunnable, timeoutHandlerPeriod);
    }

    /**
     * Returns boolean depending on {@link #transactionInProgress}
     * <p>
     * Has side effects, invokes {@link TransactionLifecycleDelegate#backPressedTransactionInProgress()}
     * if transaction is in progress.
     *
     * @return the boolean
     */
    boolean isTransactionInProgress() {
        if (transactionInProgress) {
            sharedLifecycleDelegate.backPressedTransactionInProgress();
        }

        return transactionInProgress;
    }

    void setTransactionInProgress(boolean transactionInProgress) {
        this.transactionInProgress = transactionInProgress;
    }


    interface RedirectResolverRunnable {
        void run(ResolvedUrlResult resolve, @Nullable String request, RedirectResolver redirectResolver, String inputParam);
    }

}
