package hr.webtech.pay2.view;

import android.support.annotation.NonNull;

import hr.webtech.pay2.api.PgwApi;
import hr.webtech.pay2.data.ApiError;
import hr.webtech.pay2.data.ApiErrors;
import hr.webtech.pay2.data.ShowOrderResponse;
import hr.webtech.pay2.data.TransactionSuccess;
import hr.webtech.pay2.util.DeprecationError;
import hr.webtech.pay2.util.Objects;
import hr.webtech.pay2.util.Utils;
import rx.Subscription;
import rx.android.schedulers.AndroidSchedulers;
import rx.functions.Action1;
import rx.schedulers.Schedulers;
import rx.subscriptions.CompositeSubscription;

/**
 * Created by jasminsuljic on 20/12/16.
 * pgw-android
 */
public final class TransactionLifecycleAdapter {
    /**
     * The Pgw api.
     */
    private final PgwApi pgwApi;
    /**
     * The Order number.
     */
    private final String orderNumber;
    /**
     * The Composite subscription.
     */
    private final CompositeSubscription compositeSubscription;
    /**
     * The Merchant key.
     */
    private final String merchantKey;
    @NonNull
    private final ResponseType responseType;

    @NonNull
    private final TransactionLifecycleDelegate delegate;

    /**
     * @param orderNumber  order number
     * @param pgwApi       pgw api instance
     * @param merchantKey  merchant key, can be found under merchant profile -> api settings
     * @param responseType response type instance
     * @param delegate     {@link TransactionLifecycleDelegate} implementation
     */
    public TransactionLifecycleAdapter(@NonNull String orderNumber, @NonNull PgwApi pgwApi, @NonNull String merchantKey, @NonNull ResponseType responseType, @NonNull TransactionLifecycleDelegate delegate) {
        Objects.requireNonNull(orderNumber, "orderNumber == null");
        Objects.requireNonNull(pgwApi, "pgwApi == null");
        Objects.requireNonNull(merchantKey, "merchantKey == null");
        Objects.requireNonNull(delegate, "delegate == null");

        this.delegate = delegate;
        this.merchantKey = merchantKey;
        this.responseType = responseType;
        this.pgwApi = pgwApi;
        this.orderNumber = orderNumber;
        compositeSubscription = new CompositeSubscription();
    }

    /**
     * Instantiates a new Transaction lifecycle adapter.
     *
     * @param orderNumber  the order number
     * @param pgwApi       the pgw api
     * @param merchantKey  MerchantKey, found in ipg dashboard
     * @param responseType Type of failure handling in {@link #timeout()}
     * @deprecated Use {@link TransactionLifecycleAdapter#TransactionLifecycleAdapter(String, PgwApi, String, ResponseType, TransactionLifecycleDelegate)}
     */
    @SuppressWarnings("unused")
    @Deprecated
    public TransactionLifecycleAdapter(@NonNull String orderNumber, @NonNull PgwApi pgwApi, @NonNull String merchantKey, @NonNull ResponseType responseType) throws DeprecationError {
        throw new DeprecationError("Use TransactionLifecycleAdapter#TransactionLifecycleAdapter(String, PgwApi, String, ResponseType, TransactionLifecycle)");
    }

    /**
     * Use {@link TransactionLifecycleAdapter#TransactionLifecycleAdapter(String, PgwApi, String, ResponseType)}
     *
     * @param orderNumber the order number
     * @param pgwApi      the pgw api
     * @param merchantKey the merchant key
     */
    @SuppressWarnings("unused")
    @Deprecated
    public TransactionLifecycleAdapter(@NonNull String orderNumber, @NonNull PgwApi pgwApi, @NonNull String merchantKey) throws DeprecationError {
        throw new DeprecationError("Use TransactionLifecycleAdapter#TransactionLifecycleAdapter(String, PgwApi, String, ResponseType, TransactionLifecycle)");
    }

    /**
     * Invokes {@link TransactionLifecycleDelegate#stopWebViewLoad()} and checks current order status
     * via {@link PgwApi#checkOrderId(String, String)} after {@link WebtehWebView} loading timeout.
     */
    public final void timeout() {
        delegate.logTransactionInfo("TransactionLifecycleAdapter#timeout handling");

        delegate.logTransactionInfo("delegate#stopWebViewLoad");
        delegate.stopWebViewLoad();

        Subscription subscribe;
        if (responseType == ResponseType.ShowOrderResponse) {
            subscribe = pgwApi.checkOrderId(orderNumber, Utils.sha1Hash(merchantKey + orderNumber)).subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Action1<ShowOrderResponse>() {
                        @Override
                        public void call(ShowOrderResponse showOrderResponse) {
                            delegate.logTransactionInfo(String.format("Check order id success %s", showOrderResponse));
                            delegate.checkOrderIdSuccess(showOrderResponse);
                        }
                    }, new Action1<Throwable>() {
                        @Override
                        public void call(Throwable throwable) {
                            //noinspection ThrowableResultOfMethodCallIgnored
                            final ApiError apiError = throwable instanceof ApiError ? (ApiError) throwable : ApiErrors.libraryUnknownError(throwable);
                            delegate.logTransactionInfo(String.format("Check order id failure %s", apiError));
                            delegate.handleTransactionError(apiError);
                        }
                    });
        } else {
            subscribe = pgwApi.checkOrder(orderNumber, Utils.sha1Hash(merchantKey + orderNumber)).subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Action1<TransactionSuccess>() {
                        @Override
                        public void call(TransactionSuccess transactionSuccess) {
                            delegate.logTransactionInfo(String.format("Check order success %s", transactionSuccess));
                            delegate.transactionSuccess(transactionSuccess);
                        }
                    }, new Action1<Throwable>() {
                        @Override
                        public void call(Throwable throwable) {
                            //noinspection ThrowableResultOfMethodCallIgnored
                            final ApiError apiError = throwable instanceof ApiError ? (ApiError) throwable : ApiErrors.libraryUnknownError(throwable);
                            delegate.logTransactionInfo(String.format("Check order failure %s", apiError));
                            delegate.handleTransactionError(apiError);
                        }
                    });
        }

        compositeSubscription.add(subscribe);
    }


    @SuppressWarnings("WeakerAccess")
    @NonNull
    public TransactionLifecycleDelegate getDelegate() {
        return delegate;
    }

    void onDestroy() {
        compositeSubscription.clear();
    }


    /**
     * Describes type of handling used in {@link TransactionLifecycleAdapter#timeout()} to resolve
     * transaction success.
     */
    @SuppressWarnings("WeakerAccess")
    public enum ResponseType {
        /**
         * {@link PgwApi#checkOrderId(String, String)} will be used
         */
        ShowOrderResponse,
        /**
         * {@link PgwApi#checkOrder(String, String)} will be used
         */
        TransactionSuccess
    }
}
