package hr.webtech.pay2.view;

import android.content.Context;
import android.content.res.Configuration;
import android.os.Build;
import android.support.annotation.NonNull;
import android.util.AttributeSet;
import android.util.Log;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import hr.webtech.pay2.api.PgwModule;
import hr.webtech.pay2.data.AddCardModel;
import hr.webtech.pay2.data.AddPaymentMethodModel;
import hr.webtech.pay2.data.PayCvvModel;
import hr.webtech.pay2.util.Base64Resolver;
import hr.webtech.pay2.util.InputResolver;
import hr.webtech.pay2.util.Objects;

/**
 * Created by jasminsuljic on 17/12/16.
 * pgw-android
 */
public class WebtehWebView extends WebView {

    private static final String TAG = WebtehWebView.class.getSimpleName();
    private RedirectResolver redirectResolver;
    private boolean initialized;

    private InputResolver inputResolver;
    private boolean debug;
    private long timeout;
    private SharedLifecycleDelegate delegate;

    private List<Runnable> destroyHooks = new ArrayList<>();

    /**
     * Instantiates a new Webteh web view.
     *
     * @param context the context
     */
    public WebtehWebView(Context context) {
        super(context);
    }

    /**
     * Instantiates a new Webteh web view.
     *
     * @param context the context
     * @param attrs   the attrs
     */
    public WebtehWebView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    final public void setLocale(Locale locale) {
        Log.d(TAG, String.format("setting locale %s", locale));
        final Context context = this.getContext();
        Configuration config = context.getResources().getConfiguration();
        Locale.setDefault(locale);
        config.locale = locale;
        context.getResources().updateConfiguration(config, context.getResources().getDisplayMetrics());
    }

    /**
     * Initialize.
     *
     * @param transactionLifecycleAdapter the transaction lifecycle
     * @param debug                       the debug
     * @param timeout                     Time in milliseconds after which timeout handling for started transaction will execute
     * @deprecated Use {@link #initialize(boolean, long)} and {@link #addCard(TransactionLifecycleAdapter, AddCardModel)} or {@link #addPaymentMethod(AddPaymentMethodLifecycleAdapter, AddPaymentMethodModel)}
     */
    @Deprecated
    public final void initialize(@NonNull final TransactionLifecycleAdapter transactionLifecycleAdapter, boolean debug, long timeout) {
        this.debug = debug;
        this.delegate = transactionLifecycleAdapter.getDelegate();

        final Gson gson = new Gson();
        final Base64Resolver base64Resolver = new Base64Resolver(gson);

        this.inputResolver = base64Resolver;
        this.redirectResolver = new RedirectResolver(transactionLifecycleAdapter, base64Resolver, gson, timeout);
        setupWebView();

        destroyHooks.add(new Runnable() {
            @Override
            public void run() {
                transactionLifecycleAdapter.onDestroy();
            }
        });
    }

    /**
     * Initialize.
     *
     * @param debug   the debug
     * @param timeout Time in milliseconds after which timeout handling for started transaction will execute
     */
    public final WebtehWebView initialize(boolean debug, long timeout) {
        this.debug = debug;
        this.timeout = timeout;
        this.inputResolver = new Base64Resolver(new Gson());
        setupWebView();
        return this;
    }

    /**
     * Sets web view.
     */
    final void setupWebView() {

        if (initialized) {
            if (debug) {
                Log.e(TAG, "Already initialized", new RuntimeException());
            }
            return;
        }

        WebSettings websettings = getSettings();

        websettings.setJavaScriptEnabled(true);
        websettings.setAllowContentAccess(true);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT && debug) {
            WebView.setWebContentsDebuggingEnabled(true);
        }

        clearCache(true);
        clearHistory();

        setWebViewClient(new WebViewClient() {

            @Override
            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                if (debug) {
                    Log.d(TAG, "shouldOverrideUrlLoading: " + url);
                }
                redirectResolver.notifyDelegate(redirectResolver.resolve(url), url);
                return false;
            }
        });

        setWebChromeClient(new WebChromeClient());

        initialized = true;
    }

    @Override
    public final void loadUrl(String url) {
        if (debug) {
            Log.d(TAG, "loadUrl: " + url);
        }
        super.loadUrl(url);
    }

    private void requireInitialized() {
        if (!initialized) {
            throw new IllegalStateException("WebTehWebView not initialized, initialize method not called!");
        }

        if (redirectResolver == null) {
            throw new IllegalStateException("RedirectResolver == null, initialize method not called!");
        }

        if (delegate == null) {
            throw new IllegalStateException("delegate == null, initialize method not called!");
        }
    }

    /**
     * Initializes {@link #redirectResolver} and adds destroy hook to the {@link #destroyHooks} invoking {@link TransactionLifecycleAdapter#onDestroy()}
     *
     * @param adapter
     */
    private void setupRedirectResolver(final TransactionLifecycleAdapter adapter) {
        Objects.requireNonNull(adapter);
        redirectResolver = new RedirectResolver(adapter, inputResolver, new Gson(), timeout);
        this.delegate = adapter.getDelegate();
        destroyHooks.add(new Runnable() {
            @Override
            public void run() {
                adapter.onDestroy();
            }
        });
    }

    /**
     * Initializes {@link #redirectResolver} and adds destroy hook to the {@link #destroyHooks} invoking {@link AddPaymentMethodLifecycleAdapter#onDestroy()}
     *
     * @param adapter
     */
    private void setupRedirectResolver(final AddPaymentMethodLifecycleAdapter adapter) {
        Objects.requireNonNull(adapter);
        redirectResolver = new RedirectResolver(adapter, inputResolver, new Gson(), timeout);
        this.delegate = adapter.getDelegate();
        destroyHooks.add(new Runnable() {
            @Override
            public void run() {
                adapter.onDestroy();
            }
        });
    }

    /**
     * Add card.
     *
     * @param addCardModel the add card model
     * @deprecated Use {@link #addCard(TransactionLifecycleAdapter, AddCardModel)}
     */
    @Deprecated
    public final void addCard(@NonNull AddCardModel addCardModel) {
        requireInitialized();
        Objects.requireNonNull(addCardModel, "addCardModel == null");
        delegate.logTransactionInfo(String.format("Add card loaded with order number: %s", addCardModel.getOrderNumber()));
        loadUrl(String.format(PgwModule.API_URL_ADD_CARD, inputResolver.encode(addCardModel)));
    }

    /**
     * Add card.
     *
     * @param addCardModel the add card model
     */
    public final void addCard(TransactionLifecycleAdapter adapter, @NonNull AddCardModel addCardModel) {
        setupRedirectResolver(adapter);
        requireInitialized();
        Objects.requireNonNull(addCardModel, "addCardModel == null");
        delegate.logTransactionInfo(String.format("Add card loaded with order number: %s", addCardModel.getOrderNumber()));
        loadUrl(String.format(PgwModule.API_URL_ADD_CARD, inputResolver.encode(addCardModel)));
    }

    /**
     * Pay cvv.
     *
     * @param model the pay cvv model
     * @deprecated Use {@link #payCvv(TransactionLifecycleAdapter, PayCvvModel)}
     */
    public final void payCvv(@NonNull PayCvvModel model) {
        Objects.requireNonNull(model, "model == null");
        requireInitialized();
        delegate.logTransactionInfo(String.format("Pay cvv loaded with order number: %s", model.getOrderNumber()));
        loadUrl(String.format(PgwModule.API_URL_PAY_CVV, inputResolver.encode(model)));
    }

    public final void payCvv(TransactionLifecycleAdapter adapter, @NonNull PayCvvModel model) {
        setupRedirectResolver(adapter);
        Objects.requireNonNull(model, "model == null");
        requireInitialized();
        delegate.logTransactionInfo(String.format("Pay cvv loaded with order number: %s", model.getOrderNumber()));
        loadUrl(String.format(PgwModule.API_URL_PAY_CVV, inputResolver.encode(model)));
    }

    public final void addPaymentMethod(AddPaymentMethodLifecycleAdapter adapter, AddPaymentMethodModel model) {
        setupRedirectResolver(adapter);
        requireInitialized();
        delegate.logTransactionInfo(String.format("AddPaymentMethod loaded with order number: %s", model.getOrderNumber()));
        loadUrl(String.format(PgwModule.API_URL_ADD_PAYMENT_METHOD, inputResolver.encode(model)));
    }

    /**
     * Is back allowed boolean.
     *
     * @return the boolean
     */
    public boolean isBackAllowed() {
        return !redirectResolver.isTransactionInProgress();
    }

    /**
     * On destroy.
     */
    public void onDestroy() {
        for (Runnable destroyHook : destroyHooks) {
            destroyHook.run();
        }

        destroyHooks.clear();
    }


    /**
     * The enum Resolved url result.
     */
    enum ResolvedUrlResult {
        /**
         * Unknown resolved url result.
         */
        UNKNOWN,
        /**
         * Started transaction resolved url result.
         */
        STARTED_TRANSACTION,
        ADD_PAYMENT_METHOD_STARTED_TRANSACTION,
        /**
         * Transaction success resolved url result.
         */
        TRANSACTION_SUCCESS,
        /**
         * Timeout resolved url result.
         */
        TIMEOUT,
        /**
         * Three ds resolved url result.
         */
        THREE_DS,
        ADD_PAYMENT_METHOD_THREE_DS,

        STARTED_TRANSACTION_3DS,
        ADD_PAYMENT_METHOD_STARTED_TRANSACTION_3DS,
        /**
         * Transaction failure resolved url result.
         */
        TRANSACTION_FAILURE,

        /**
         * Add payment method success resolved url result
         */
        ADD_PAYMENT_METHOD_SUCCESS,

        /**
         * Add payment method failure url result
         */
        ADD_PAYMENT_METHOD_FAILURE
    }


}

